#!/usr/bin/env python3
import subprocess
import os
import datetime
import socket
import traceback
import sys
import random
import string
'''This script is a Python script that performs several functions, such as
getting the date and time for Gemini, getting the hostname of the device,
getting the LAN IP address of the device, calling update_slaves function,
getting the external IP of the device, getting the disk usage, getting the
crontab text, writing some files for the GMI and defining a dictionary of
ports.

The subprocess library is used to execute UNIX commands like rsync, df, and
cron, while the os library is used to interact with the operating system,
for example, to get the hostname. The datetime library provides utilities to
convert between strings and datetime objects while the socket library is used
for networking. The traceback library is used to print stack trace information,
while sys is used to get information about Python runtime environment.
'''
'''
The script defines a dictionary svrdict, which stores details of the servers,
i.e., au and gmi.
'''
svrdict = {"hk": "/var/gmi/hk.longtext.space/inf/",
    "rp": "/home/identity2/public_gemini/inf/",
    "gmi": "/var/gmi/gmi.longtext.space/inf/",
    "ep": "/home/identity2/public_gemini/inf/"}  # ,"le":2011,"hu":201
    #    FAIL
#     ,
#     "ep": "/home/identity2/public_gemini/inf/",
#     "rp": "/home/identity2/public_gemini/inf/"
#   "au": "/var/gmi/au.longtext.space/inf/",

# Get hostname
host = socket.gethostname() #hostname le,hu (pc) cn,hk,au (rpi)
if "envs.net" in host:
    host = "ep"
elif "rawtext.club" in host:
    host = "rp"


print(svrdict.get(f"{host}"))

# Get date and time for Gemini
get_datet = datetime.datetime.now() #gpresent time in datetime format
dt = get_datet.strftime("%y-%m-%d  %H:%M  +8") #Formating date time for Gemini



# Get LAN IP address
def get_lan_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('10.255.255.255', 1))
        ip = s.getsockname()[0]
        ip = f"Lan {ip.replace('192.168.0.','')} "
    except Exception as e:
        ip = f"Get IP failed {e}"
    finally:
        s.close()
    return ip        
#update()

# Call update_slaves function
def update_slaves():
    report = "Feedback on Bin Code update:"
    for key, value in svrdict.items():
        try:
            start = "/usr/bin/rsync -avz /b/croncollect.py"
            bin_dir = "/b/" if key != "ep" and key != "rp" \
                      else "/home/identity2/b/"
            cmd = f"{start} {key[0]}:{bin_dir}croncollect.py"
            p = subprocess.run(cmd, shell=True, capture_output=True)
            output = p.stdout.decode("utf-8")
            report = f"{report} ● {key.upper()} ●   {output}"
        except Exception as e:
            report = f"{report}Slave update ● {key.upper()} ● failed as: {e}"       
# update_slaves()            
            
            
# Get my external IP
'''
Another function, ip_ext(), gets the external IP address of a device.
The disk_use() function gets the disk usage on the device. For querying the
crontab text on a device, the cron_l() function is called. The art dictionary
contains ports and their relative positions.
'''

def ip_ext():
    try:
        e = ('/usr/bin/dig @resolver4.opendns.com myip.opendns.com +short')
        output = subprocess.run(
            f'{e}', shell=True, capture_output=True, text=True).stdout.strip()
        ipr = output.strip().replace('.','O').replace('1','.').replace('2',':') 
        ipr = f"{''.join(random.choices('123456789:.', k=5))}1{ipr}"
    except Exception as e:
        ipr = f"Dig {host} fail {e}"
    return ipr

# Get the disk usage
def disk_use():
    try:
        o = subprocess.check_output(['df', '-h', '/'], shell=False)
        o = o.splitlines()
        disk = ''
        for line in o[1:]:   # o[1:]:  loop array(exluding header details)
            result = line.decode().split() #array "o" on comma seperator
            disk = f"{disk}{result[0]} {result[1]}, {result[3]} Free"
            disk = disk.replace('/dev/nvme0n1','')
    except Exception as e: 
            disk = f"Disk usage Error: {e}"
    return disk

# Get ctontab text

def cron_l():
    try:
        output = subprocess.run(['crontab', '-l'], capture_output=True, text=True).stdout
        cronu = output.replace('#',' ○ ╳ ')
        cronu = f""" ░●░░●░░░░░░░░░░░░░░░░ {host.upper()} ░░░░░░░░░░░░░░░░●░░●░
                {cronu}"""
        with open(f"{svrdict.get(host)}cron-{host}.gmi", "w") as f:
            f.write(cronu)
    except Exception as e: 
        print(f"Cron fetch error: {e}")
        cronu = ""
    return cronu
cron_l()

# write some files for the GMI ascii images  
art = {
    'defau': ' ● ○ ○ ○  D E F A U L T  ○  P O R T  ○ ○ ○ ●',
    'sport': ' ● ○ ○ ○  C U S T O M  ○  P O R T S  ○ ○ ○ ●',
    'local': ' ● ○ ○ L I N K  T O  L O C A L  F I L E ○ ○ ●',
    'thend': ' ● ○ ○ ○ ○ ○ ○ ○ ○ ○ ○ ○ ○ ○ ○ ○ ○ ○ ●',
    'cronr': ' ● ○ ○ ○ ○  C R O N  ○  R O O T  ○ ○ ○ ○ ●',
    'cronu': ' ● ○ ○ ○ ○  C R O N  ○  U S E R  ○ ○ ○ ○ ●',
    'links': ' ● ○ ○ ○ ○  T E X T  O F  F I L E  ○ ○ ○ ○ ●'
}
for key, value in art.items():
    a = open(f"{svrdict.get(host)}{key}", "w")
    a.write(value)
    a.close()

        
# art-cronr.gmi art-cronu.gmi art-thend.gmi


#>>>>>>>>>>>>> index.gmi  Will be over written on HK machines
           
f = open(f"{svrdict.get(host)}ind.gmi", "w")
ind = f"""
``` language-ascii-art2\n{art['local']}\n```
=> data-cn.gmi   CN
=> data-hk.gmi   HK
=> data-au.gmi   AU
=> data-gmi.gmi  GMI
=> data-rp.gmi   RP
=> data-ep.gmi   EP
``` language-ascii-art2\n{art['links']}\n```
"""
f.write(ind)

#>>>>>>>>>>>>> ind.gmi  used to rewrite index at destination
f = open(f"{svrdict.get(host)}index.gmi", "w")
f.write(ind)
f.close() 
 
o = f"""# {host}
> Shell environ['SHELL']
> {dt}
> {disk_use()}"""
f = open(f"{svrdict.get(host)}data-{host}.gmi", "w")
# Local copy
f.write(o)
f.close()

# Rsync the data directories
def do_rsync():
    try:
        output_str = ''
        for key, value in svrdict.items():
            from_dir = f"{value}" if key != "ep" and key != "rp" else \
                                "/home/identity2/public_gmi/"
            cmd = f'/usr/bin/rsync -avz {from_dir} {key[0]}:{value}'
            compl = subprocess.run(cmd,
                                shell=True, capture_output=True, text=True)
            output_str += f'\nSuccess ●{key.upper()}● \n{compl.stdout}\n'
    except Exception as e:
        output_str += f'\n \rsyncFail ●{key.upper()}●\n{e}'
    return output_str
#do_rsync()
